@@ -10,12 +10,19 @@
 #' @return An `R6Class` object of class `Taxon`
 #' @family classes
 #' @examples
-#' (x <- taxon(
+#' (a <- taxon(
 #'   name = taxon_name("Poa annua"),
 #'   rank = taxon_rank("species"),
 #'   id = taxon_id(93036)
 #' ))
+#' taxa(a, a, a)
+#'
+#' # a null set
+#' x <- taxon(NULL)
 #' taxa(x, x, x)
+#'
+#' # combo non-null and null
+#' taxa(a, x, a)
 taxa <- function(..., .list = NULL) {
   tt <- get_dots_or_list(..., .list = .list)
   if (!all(vapply(tt, inherits, logical(1), what = "Taxon"))) {
@@ -30,13 +37,21 @@ print.taxa <- function(x, ...) {
   cat("<taxa>", "\n")
   cat("  no. taxa: ", length(x), "\n")
   if (length(x)) {
-    for (i in seq_along(x[1:min(10, length(x))])) {
-      cat(
-        sprintf("  %s / %s / %s",
-                x[[i]]$name$name %||% "",
-                x[[i]]$rank$name %||% "",
-                x[[i]]$id$id %||% ""
-        ), "\n")
+    if (all(vapply(x, function(z) z$is_empty(), logical(1)))) {
+      cat("   empty set", "\n")
+    } else {
+      for (i in seq_along(x[1:min(10, length(x))])) {
+        if (x[[i]]$is_empty()) {
+          cat("  empty", "\n")
+        } else {
+          cat(
+            sprintf("  %s / %s / %s",
+                    x[[i]]$name$name %||% "",
+                    x[[i]]$rank$name %||% "",
+                    x[[i]]$id$id %||% ""
+            ), "\n")
+        }
+      }
     }
   }
   if (length(x) > 10) cat("  ...")
